(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.Formini = {}));
}(this, (function (exports) { 'use strict';

  var defaultMapper = function defaultMapper() {
    return {};
  };

  var concat = Array.prototype.concat.bind([]);
  var objectAssign = Object.assign;
  var defineProperty = function defineProperty(obj, key, value) {
    obj[key] = value;
    return obj;
  };
  var applyFunction = function applyFunction(func, args) {
    return func.apply(undefined, args);
  };
  var debounce = function debounce(func, wait) {
    var timeout;
    return function () {
      var args = arguments;
      clearTimeout(timeout);
      timeout = setTimeout(function () {
        applyFunction(func, args);
      }, wait);
      return timeout;
    };
  };
  var arr2map = function arr2map(arr, mapper) {
    return applyFunction(objectAssign, concat({}, (arr || []).map(mapper || defaultMapper)));
  };
  var mapObject = function mapObject(obj, mapper) {
    return applyFunction(objectAssign, concat({}, Object.entries(obj || {}).map(function (args, idx) {
      return (mapper || defaultMapper).apply(undefined, concat(args, idx));
    })));
  };
  var fields2map = function fields2map(fields, getFieldValue) {
    return arr2map(fields, function (field) {
      return defineProperty({}, field.name, getFieldValue(field));
    });
  };
  var getFieldsData = function getFieldsData(fields) {
    return {
      keys: fields.map(function (field) {
        return field.name;
      }),
      allRules: fields2map(fields, function (field) {
        return field.rules;
      }),
      values: fields2map(fields, function (field) {
        return field.defaultValue;
      }),
      errors: fields2map(fields, function () {
        return undefined;
      })
    };
  };
  var getEventListener = function getEventListener(listener, debounceDelay) {
    var debouncedListener = debounce(listener, debounceDelay);
    return debounceDelay !== false ? debouncedListener : listener;
  };
  var createEventBus = function createEventBus() {
    var listenerMap = {};

    var getCollection = function getCollection(event) {
      listenerMap[event] = listenerMap[event] || new Set();
      return listenerMap[event];
    };

    return {
      on: function on(event, listener) {
        var collection = getCollection(event);
        collection.add(listener);
        return function () {
          return collection.delete(listener);
        };
      },
      emit: function emit(event, value) {
        var collection = getCollection(event);
        Array.from(collection.values()).forEach(function (listener) {
          return listener(value);
        });
      }
    };
  };
  var isFunction = function isFunction(val) {
    return typeof val === 'function';
  };

  var eventBus = createEventBus();
  var collection = new Set();
  var reactiveMap = new Map();

  var depend = function depend(effect) {
    collection.clear();
    var res = effect();
    var dependKeys = Array.from(collection.values());
    collection.clear(); // console.log(dependKeys)

    return [res, dependKeys];
  };

  var watch = function watch(watcher, callback, lazy) {
    var canTrigger = true;
    var currentValue;
    var listenerMap = {};

    var updateDependKey = function updateDependKey(eventKey) {
      if (listenerMap[eventKey]) {
        return;
      }

      listenerMap[eventKey] = eventBus.on(eventKey, function () {
        // console.log('on event', eventKey)
        canTrigger = true;

        if (!lazy) {
          trigger();
        }
      });
    };

    function effect() {
      if (reactiveMap.has(watcher)) {
        updateDependKey(reactiveMap.get(watcher));
        currentValue = watcher;
        return watcher;
      }

      var dependConfig = depend(watcher);
      var value = dependConfig[0];
      var dependKeys = dependConfig[1];
      dependKeys.forEach(updateDependKey);
      currentValue = value;
      return value;
    }

    function trigger() {
      if (!canTrigger) {
        return currentValue;
      } // console.log('compute 了')


      var value = effect();
      callback(value);
      canTrigger = false;
      return value;
    }

    function stop() {
      Object.values(listenerMap).forEach(function (stop) {
        return stop();
      });
    }

    effect();
    stop.trigger = trigger;
    return stop;
  };
  var computed = function computed(effect, immediate) {
    var computedValue;

    var _watch = watch(effect, function (value) {
      computedValue = value;
    }, !immediate),
        compute = _watch.trigger;

    var reactiveComputed = {};
    Object.defineProperty(reactiveComputed, 'value', {
      get: function get() {
        return compute();
      },
      set: function set() {
        return computedValue;
      }
    });
    return reactiveComputed;
  };

  var getRandomEventKey = function getRandomEventKey() {
    return Math.floor(Math.random() * 100000);
  };

  var reactive = function reactive(obj) {
    var objEventKey = getRandomEventKey();

    var addEventCollection = function addEventCollection(prop) {
      collection.add("".concat(objEventKey, ":").concat(String(prop)));
    };

    var emitChangeEvent = function emitChangeEvent(prop) {
      eventBus.emit("".concat(objEventKey, ":").concat(String(prop)));
      eventBus.emit(objEventKey);
    };

    var reactiveObj = new Proxy(objectAssign({}, obj), {
      get: function get(obj, prop) {
        // console.log(`get ${prop}`)
        addEventCollection(prop);
        return obj[prop];
      },
      has: function has(obj, prop) {
        // console.log(`check has ${prop}`)
        addEventCollection(prop);
        return prop in obj;
      },
      set: function set(obj, prop, value) {
        var changed = value !== obj[prop]; // console.log(`set ${prop} to ${value}`)

        obj[prop] = value;

        if (changed) {
          emitChangeEvent(prop);
        }

        return true;
      },
      deleteProperty: function deleteProperty(obj, prop) {
        var changed = (prop in obj);
        delete obj[prop];

        if (changed) {
          emitChangeEvent(prop);
        }

        return true;
      }
    });
    reactiveMap.set(reactiveObj, objEventKey);
    return reactiveObj;
  }; // Object.assign(window, {
  //   reactive,
  //   computed,
  //   watch,
  // })
  // var obj = reactive({ a: 1, b: 1, c: 4, d: 5 })
  // var aa = computed(() => obj.a * 10)
  // var bb = computed(() => obj.b * 10, true)
  // var stopC = watch(
  //   () => obj.c,
  //   () => {
  //     console.log(obj.c)
  //   }
  // )
  // var stop = watch(obj, () => {
  // console.log('obj 更新了')
  // })

  var RELATIVE_CHANGE = 'rc';
  function createForm(formOptions) {
    formOptions = formOptions || {};
    var initalFields = formOptions.fields || [];
    var initialRelativeConfigs = formOptions.relatives || {};
    var initialFieldsData = getFieldsData(initalFields);
    var keys = initialFieldsData.keys,
        allRules = initialFieldsData.allRules;

    var setReactivityData = function setReactivityData(data, update) {
      objectAssign(data, update); // 过滤掉未在 fields 内声明的数据

      Object.keys(data).forEach(function (key) {
        if (!keys.includes(key)) {
          delete data[key];
        }
      });
    }; // 创建响应式数据相关函数


    var buildReactivityData = function buildReactivityData(sourceData) {
      var values = reactive(sourceData);

      var getValue = function getValue(key) {
        return (values || {})[key];
      };

      var getValues = function getValues() {
        return objectAssign({}, values);
      };

      var setValue = function setValue(key, value) {
        return setReactivityData(values, defineProperty({}, key, value));
      };

      var setValues = function setValues(update) {
        return setReactivityData(values, update || {});
      };

      var watchValues = function watchValues(listener, options) {
        return watch(values, getEventListener(listener, (options || {}).debounce));
      };

      var watchValue = function watchValue(key, listener, options) {
        return watch(function () {
          return values[key];
        }, getEventListener(listener, (options || {}).debounce));
      };

      return [values, getValue, getValues, setValue, setValues, watchValues, watchValue];
    }; // +++++++++++++++++++++++ values 相关 begin ++++++++++++++++++++++++++++


    var valueDataConfig = buildReactivityData(initialFieldsData.values);
    var values = valueDataConfig[0];
    var getValue = valueDataConfig[1];
    var getValues = valueDataConfig[2];
    var setValue = valueDataConfig[3];
    var setValues = valueDataConfig[4];
    var watchValues = valueDataConfig[5];
    var watchValue = valueDataConfig[6]; // ----------------------- values 相关 end ------------------------------
    // +++++++++++++++++++++++ errors 相关 begin ++++++++++++++++++++++++++++

    var errorDataConfig = buildReactivityData(initialFieldsData.errors);
    var errors = errorDataConfig[0];
    var getError = errorDataConfig[1];
    var getErrors = errorDataConfig[2];
    var setError = errorDataConfig[3];
    var setErrors = errorDataConfig[4];
    var watchErrors = errorDataConfig[5];
    var watchError = errorDataConfig[6]; // ----------------------- errors 相关 end ------------------------------
    // +++++++++++++++++++++++ fields 相关 begin ++++++++++++++++++++++++++++

    var fields = concat(initalFields);

    var getFields = function getFields() {
      return concat(fields);
    };

    var setFields = function setFields(nextFields) {
      fields = concat(nextFields || []);
      var fieldsData = getFieldsData(fields);
      keys = fieldsData.keys;
      allRules = fieldsData.allRules;
      setValues(objectAssign({}, fieldsData.values, values));
      setErrors(objectAssign({}, fieldsData.errors, errors));
    };

    var removeField = function removeField(fieldName) {
      return setFields(getFields().filter(function (field) {
        return field.name !== fieldName;
      }));
    };

    var addField = function addField(field) {
      setFields(concat(getFields(), field));
      return function () {
        return removeField(field.name);
      };
    }; // ----------------------- fields 相关 end ------------------------------
    // +++++++++++++++++++++++ relatives 相关 begin ++++++++++++++++++++++++++++


    var eventBus = createEventBus();
    var triggerRelativesChange = debounce(function () {
      return eventBus.emit(RELATIVE_CHANGE, getRelatives());
    });
    var currentRelatives = {};

    var buildRelatives = function buildRelatives(configs) {
      return mapObject(configs, function (key, compute) {
        return defineProperty({}, key, computed(function () {
          // console.log('compute key:', key)
          var currentRelative = currentRelatives[key];
          var nextValue = compute(values, errors);

          if (nextValue !== currentRelative) {
            eventBus.emit("".concat(RELATIVE_CHANGE, ":").concat(key), nextValue);
            triggerRelativesChange();
          }

          currentRelatives[key] = nextValue;
          return nextValue;
        }, true));
      }) || {};
    };

    var currentRelativeConfigs = objectAssign({}, initialRelativeConfigs);
    var relatives = buildRelatives(currentRelativeConfigs);

    var getRelative = function getRelative(key) {
      return (relatives[key] || {}).value;
    };

    var getRelatives = function getRelatives() {
      return mapObject(relatives, function (key, relative) {
        return defineProperty({}, key, relative.value);
      });
    };

    var setRelatives = function setRelatives(configs) {
      currentRelativeConfigs = objectAssign({}, configs);
      relatives = buildRelatives(configs);
    };

    var removeRelative = function removeRelative(name) {
      delete currentRelativeConfigs[name];
      setRelatives(currentRelativeConfigs);
    };

    var addRelative = function addRelative(name, compute) {
      setRelatives(objectAssign({}, currentRelativeConfigs, defineProperty({}, name, compute)));
      return function () {
        return removeRelative(name);
      };
    };

    var triggerRelativeWatcher = function triggerRelativeWatcher(eventName, listener, options) {
      var eventListener = getEventListener(listener, (options || {}).debounce);
      return eventBus.on(eventName, eventListener);
    };

    var watchRelatives = function watchRelatives(listener, options) {
      return triggerRelativeWatcher(RELATIVE_CHANGE, listener, options);
    };

    var watchRelative = function watchRelative(key, listener, options) {
      return triggerRelativeWatcher("".concat(RELATIVE_CHANGE, ":").concat(key), listener, options);
    }; // ----------------------- relatives 相关 end ------------------------------
    // +++++++++++++++++++++++ 其他 ++++++++++++++++++++++++++++


    var watch$1 = function watch(listener) {
      var eventListender = getEventListener(listener);
      var debounceConfig = {
        debounce: false
      };
      var stopValuesListener = watchValues(eventListender, debounceConfig);
      var stopErrorsListener = watchValues(eventListender, debounceConfig);
      var stopRelativesListener = watchRelatives(eventListender, debounceConfig);
      return function () {
        stopValuesListener();
        stopErrorsListener();
        stopRelativesListener();
      };
    };

    var validate = function validate(filedKeys) {
      filedKeys = filedKeys || keys;

      if (typeof filedKeys === 'string') {
        filedKeys = [filedKeys];
      } // 多个 fields 并行校验
      // 单个 field 的多个 rules 串行校验


      return Promise.all(filedKeys.map(function (key) {
        var rules = allRules[key] || [];
        var value = values[key];
        errors[key] = undefined;

        var applyRule = function applyRule(rule) {
          return Promise.resolve(rule(value, values)).then(function (result) {
            return (// 利用 reject 中断 Promise 串行链条
              typeof result !== 'undefined' ? Promise.reject(result) : undefined
            );
          });
        };

        return rules.filter(isFunction).reduce(function (task, rule) {
          return task.then(function () {
            return applyRule(rule);
          });
        }, Promise.resolve()).catch(function (error) {
          errors[key] = error;
          return error;
        }).then(function (error) {
          return !!error;
        });
      })).then(function (result) {
        var valid = !result.some(Boolean);
        return valid;
      });
    };

    var reset = function reset() {
      var fieldsData = getFieldsData(fields);
      setValues(fieldsData.values);
      setErrors(fieldsData.errors);
    }; // ----------------------- 其他 ----------------------------


    return {
      getValue: getValue,
      getValues: getValues,
      setValue: setValue,
      setValues: setValues,
      watchValues: watchValues,
      watchValue: watchValue,
      getError: getError,
      getErrors: getErrors,
      setError: setError,
      setErrors: setErrors,
      watchErrors: watchErrors,
      watchError: watchError,
      getFields: getFields,
      setFields: setFields,
      removeField: removeField,
      addField: addField,
      getRelative: getRelative,
      getRelatives: getRelatives,
      setRelatives: setRelatives,
      removeRelative: removeRelative,
      addRelative: addRelative,
      watchRelatives: watchRelatives,
      watchRelative: watchRelative,
      watch: watch$1,
      validate: validate,
      reset: reset
    };
  }

  exports.createForm = createForm;
  exports.default = createForm;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.umd.js.map
